@@ -64,7 +64,7 @@ void PeersAdditionDialog::validateInput()
                     QMessageBox::Ok);
         return;
     }
-    for (const QString &peer : asConst(m_ui->textEditPeers->toPlainText().trimmed().split('\n')))
+    for (const QString &peer : asConst(m_ui->textEditPeers->toPlainText().trimmed().split(u'\n')))
     {
         const BitTorrent::PeerAddress addr = BitTorrent::PeerAddress::parse(peer);
         if (!addr.ip.isNull())
