@@ -52,7 +52,7 @@ public:
         COL_PEERS,
         COL_SEEDS,
         COL_LEECHES,
-        COL_DOWNLOADED,
+        COL_TIMES_DOWNLOADED,
         COL_MSG,
 
         COL_COUNT
@@ -61,8 +61,6 @@ public:
     explicit TrackerListWidget(PropertiesWidget *properties);
     ~TrackerListWidget();
 
-    int visibleColumnsCount() const;
-
 public slots:
     void setRowColor(int row, const QColor &color);
 
@@ -72,25 +70,30 @@ public slots:
     void clear();
     void loadStickyItems(const BitTorrent::Torrent *torrent);
     void loadTrackers();
-    void askForTrackers();
     void copyTrackerUrl();
     void reannounceSelected();
     void deleteSelectedTrackers();
     void editSelectedTracker();
-    void showTrackerListMenu(const QPoint &);
-    void displayToggleColumnsMenu(const QPoint &);
+    void showTrackerListMenu();
     void loadSettings();
     void saveSettings() const;
 
 protected:
     QVector<QTreeWidgetItem *> getSelectedTrackerItems() const;
 
+private slots:
+    void openAddTrackersDialog();
+    void displayColumnHeaderMenu();
+
 private:
+    int visibleColumnsCount() const;
+    void wheelEvent(QWheelEvent *event) override;
+
     static QStringList headerLabels();
 
-    PropertiesWidget *m_properties;
+    PropertiesWidget *m_properties = nullptr;
     QHash<QString, QTreeWidgetItem *> m_trackerItems;
-    QTreeWidgetItem *m_DHTItem;
-    QTreeWidgetItem *m_PEXItem;
-    QTreeWidgetItem *m_LSDItem;
+    QTreeWidgetItem *m_DHTItem = nullptr;
+    QTreeWidgetItem *m_PEXItem = nullptr;
+    QTreeWidgetItem *m_LSDItem = nullptr;
 };
