@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2014  Vladimir Golovnev <glassez@yandex.ru>
+ * Copyright (C) 2014, 2022  Vladimir Golovnev <glassez@yandex.ru>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,6 +32,7 @@
 
 #include <QDateTime>
 #include <QDebug>
+#include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QJsonDocument>
@@ -42,7 +43,6 @@
 #include <QUrl>
 
 #include "base/algorithm.h"
-#include "base/global.h"
 #include "base/http/httperror.h"
 #include "base/logger.h"
 #include "base/preferences.h"
@@ -63,12 +63,11 @@
 #include "api/transfercontroller.h"
 
 const int MAX_ALLOWED_FILESIZE = 10 * 1024 * 1024;
-const char C_SID[] = "SID"; // name of session id cookie
+const auto C_SID = QByteArrayLiteral("SID"); // name of session id cookie
 
-const QString PATH_PREFIX_ICONS {QStringLiteral("/icons/")};
-const QString WWW_FOLDER {QStringLiteral(":/www")};
-const QString PUBLIC_FOLDER {QStringLiteral("/public")};
-const QString PRIVATE_FOLDER {QStringLiteral("/private")};
+const QString WWW_FOLDER = u":/www"_qs;
+const QString PUBLIC_FOLDER = u"/public"_qs;
+const QString PRIVATE_FOLDER = u"/private"_qs;
 
 namespace
 {
@@ -80,7 +79,7 @@ namespace
 
         for (const auto &cookie : cookies)
         {
-            const int idx = cookie.indexOf('=');
+            const int idx = cookie.indexOf(u'=');
             if (idx < 0)
                 continue;
 
@@ -93,8 +92,8 @@ namespace
 
     QUrl urlFromHostHeader(const QString &hostHeader)
     {
-        if (!hostHeader.contains(QLatin1String("://")))
-            return {QLatin1String("http://") + hostHeader};
+        if (!hostHeader.contains(u"://"))
+            return {u"http://"_qs + hostHeader};
         return hostHeader;
     }
 
@@ -102,34 +101,43 @@ namespace
     {
         contentType = contentType.toLower();
 
-        if (contentType.startsWith(QLatin1String("image/")))
-            return QLatin1String("private, max-age=604800");  // 1 week
+        if (contentType.startsWith(u"image/"))
+            return u"private, max-age=604800"_qs;  // 1 week
 
         if ((contentType == Http::CONTENT_TYPE_CSS)
             || (contentType == Http::CONTENT_TYPE_JS))
             {
             // short interval in case of program update
-            return QLatin1String("private, max-age=43200");  // 12 hrs
+            return u"private, max-age=43200"_qs;  // 12 hrs
         }
 
-        return QLatin1String("no-store");
+        return u"no-store"_qs;
+    }
+
+    QString createLanguagesOptionsHtml()
+    {
+        // List language files
+        const QDir langDir {u":/www/translations"_qs};
+        const QStringList langFiles = langDir.entryList(QStringList(u"webui_*.qm"_qs), QDir::Files);
+        QStringList languages;
+        for (const QString &langFile : langFiles)
+        {
+            const QString localeStr = langFile.section(u"_"_qs, 1, -1).section(u"."_qs, 0, 0); // remove "webui_" and ".qm"
+            languages << u"<option value=\"%1\">%2</option>"_qs.arg(localeStr, Utils::Misc::languageToLocalizedString(localeStr));
+            qDebug() << "Supported locale:" << localeStr;
+        }
+
+        return languages.join(u'\n');
     }
 }
 
-WebApplication::WebApplication(QObject *parent)
+WebApplication::WebApplication(IApplication *app, QObject *parent)
     : QObject(parent)
+    , ApplicationComponent(app)
     , m_cacheID {QString::number(Utils::Random::rand(), 36)}
+    , m_authController {new AuthController(this, app, this)}
 {
-    registerAPIController(QLatin1String("app"), new AppController(this, this));
-    registerAPIController(QLatin1String("auth"), new AuthController(this, this));
-    registerAPIController(QLatin1String("log"), new LogController(this, this));
-    registerAPIController(QLatin1String("rss"), new RSSController(this, this));
-    registerAPIController(QLatin1String("search"), new SearchController(this, this));
-    registerAPIController(QLatin1String("sync"), new SyncController(this, this));
-    registerAPIController(QLatin1String("torrents"), new TorrentsController(this, this));
-    registerAPIController(QLatin1String("transfer"), new TransferController(this, this));
-
-    declarePublicAPI(QLatin1String("auth/login"));
+    declarePublicAPI(u"auth/login"_qs);
 
     configure();
     connect(Preferences::instance(), &Preferences::changed, this, &WebApplication::configure);
@@ -143,55 +151,32 @@ WebApplication::~WebApplication()
 
 void WebApplication::sendWebUIFile()
 {
-    const QStringList pathItems {request().path.split('/', Qt::SkipEmptyParts)};
-    if (pathItems.contains(".") || pathItems.contains(".."))
+    const QStringList pathItems {request().path.split(u'/', Qt::SkipEmptyParts)};
+    if (pathItems.contains(u".") || pathItems.contains(u".."))
         throw InternalServerErrorHTTPError();
 
-    if (!m_isAltUIUsed)
-    {
-        if (request().path.startsWith(PATH_PREFIX_ICONS))
-        {
-            const QString imageFilename {request().path.mid(PATH_PREFIX_ICONS.size())};
-            sendFile(QLatin1String(":/icons/") + imageFilename);
-            return;
-        }
-    }
+    const QString path = (request().path != u"/")
+        ? request().path
+        : u"/index.html"_qs;
 
-    const QString path
-    {
-        (request().path != QLatin1String("/")
-                ? request().path
-                : QLatin1String("/index.html"))
-    };
-
-    QString localPath
-    {
-        m_rootFolder
-                + (session() ? PRIVATE_FOLDER : PUBLIC_FOLDER)
-                + path
-    };
-
-    QFileInfo fileInfo {localPath};
-
-    if (!fileInfo.exists() && session())
+    Path localPath = m_rootFolder
+                / Path(session() ? PRIVATE_FOLDER : PUBLIC_FOLDER)
+                / Path(path);
+    if (!localPath.exists() && session())
     {
         // try to send public file if there is no private one
-        localPath = m_rootFolder + PUBLIC_FOLDER + path;
-        fileInfo.setFile(localPath);
+        localPath = m_rootFolder / Path(PUBLIC_FOLDER) / Path(path);
     }
 
     if (m_isAltUIUsed)
     {
-#ifdef Q_OS_UNIX
         if (!Utils::Fs::isRegularFile(localPath))
-        {
-            status(500, "Internal Server Error");
-            print(tr("Unacceptable file type, only regular file is allowed."), Http::CONTENT_TYPE_TXT);
-            return;
-        }
-#endif
+            throw InternalServerErrorHTTPError(tr("Unacceptable file type, only regular file is allowed."));
 
-        while (fileInfo.filePath() != m_rootFolder)
+        const QString rootFolder = m_rootFolder.data();
+
+        QFileInfo fileInfo {localPath.parentPath().data()};
+        while (fileInfo.path() != rootFolder)
         {
             if (fileInfo.isSymLink())
                 throw InternalServerErrorHTTPError(tr("Symlinks inside alternative UI folder are forbidden."));
@@ -205,7 +190,7 @@ void WebApplication::sendWebUIFile()
 
 void WebApplication::translateDocument(QString &data) const
 {
-    const QRegularExpression regex("QBT_TR\\((([^\\)]|\\)(?!QBT_TR))+)\\)QBT_TR\\[CONTEXT=([a-zA-Z_][a-zA-Z0-9_]*)\\]");
+    const QRegularExpression regex(u"QBT_TR\\((([^\\)]|\\)(?!QBT_TR))+)\\)QBT_TR\\[CONTEXT=([a-zA-Z_][a-zA-Z0-9_]*)\\]"_qs);
 
     int i = 0;
     bool found = true;
@@ -226,8 +211,8 @@ void WebApplication::translateDocument(QString &data) const
             QString translation = loadedText.isEmpty() ? sourceText : loadedText;
 
             // Use HTML code for quotes to prevent issues with JS
-            translation.replace('\'', "&#39;");
-            translation.replace('\"', "&#34;");
+            translation.replace(u'\'', u"&#39;"_qs);
+            translation.replace(u'\"', u"&#34;"_qs);
 
             data.replace(i, regexMatch.capturedLength(), translation);
             i += translation.length();
@@ -237,8 +222,8 @@ void WebApplication::translateDocument(QString &data) const
             found = false; // no more translatable strings
         }
 
-        data.replace(QLatin1String("${LANG}"), m_currentLocale.left(2));
-        data.replace(QLatin1String("${CACHEID}"), m_cacheID);
+        data.replace(u"${LANG}"_qs, m_currentLocale.left(2));
+        data.replace(u"${CACHEID}"_qs, m_cacheID);
     }
 }
 
@@ -266,16 +251,25 @@ void WebApplication::doProcessRequest()
         return;
     }
 
-    const QString action = match.captured(QLatin1String("action"));
-    const QString scope = match.captured(QLatin1String("scope"));
-
-    APIController *controller = m_apiControllers.value(scope);
-    if (!controller)
-        throw NotFoundHTTPError();
+    const QString action = match.captured(u"action"_qs);
+    const QString scope = match.captured(u"scope"_qs);
 
+    // Check public/private scope
     if (!session() && !isPublicAPI(scope, action))
         throw ForbiddenHTTPError();
 
+    // Find matching API
+    APIController *controller = nullptr;
+    if (session())
+        controller = session()->getAPIController(scope);
+    if (!controller)
+    {
+        if (scope == u"auth")
+            controller = m_authController;
+        else
+            throw NotFoundHTTPError();
+    }
+
     // Filter HTTP methods
     const auto allowedMethodIter = m_allowedMethod.find({scope, action});
     if (allowedMethodIter == m_allowedMethod.end())
@@ -302,6 +296,9 @@ void WebApplication::doProcessRequest()
         case QMetaType::QJsonDocument:
             print(result.toJsonDocument().toJson(QJsonDocument::Compact), Http::CONTENT_TYPE_JSON);
             break;
+        case QMetaType::QByteArray:
+            print(result.toByteArray(), Http::CONTENT_TYPE_TXT);
+            break;
         case QMetaType::QString:
         default:
             print(result.toString(), Http::CONTENT_TYPE_TXT);
@@ -334,8 +331,7 @@ void WebApplication::configure()
     const auto *pref = Preferences::instance();
 
     const bool isAltUIUsed = pref->isAltWebUiEnabled();
-    const QString rootFolder = Utils::Fs::expandPathAbs(
-                !isAltUIUsed ? WWW_FOLDER : pref->getWebUiRootFolder());
+    const Path rootFolder = (!isAltUIUsed ? Path(WWW_FOLDER) : pref->getWebUiRootFolder());
     if ((isAltUIUsed != m_isAltUIUsed) || (rootFolder != m_rootFolder))
     {
         m_isAltUIUsed = isAltUIUsed;
@@ -344,7 +340,7 @@ void WebApplication::configure()
         if (!m_isAltUIUsed)
             LogMsg(tr("Using built-in Web UI."));
         else
-            LogMsg(tr("Using custom Web UI. Location: \"%1\".").arg(m_rootFolder));
+            LogMsg(tr("Using custom Web UI. Location: \"%1\".").arg(m_rootFolder.toString()));
     }
 
     const QString newLocale = pref->getLocale();
@@ -353,7 +349,7 @@ void WebApplication::configure()
         m_currentLocale = newLocale;
         m_translatedFiles.clear();
 
-        m_translationFileLoaded = m_translator.load(m_rootFolder + QLatin1String("/translations/webui_") + newLocale);
+        m_translationFileLoaded = m_translator.load((m_rootFolder / Path(u"translations/webui_"_qs) + newLocale).data());
         if (m_translationFileLoaded)
         {
             LogMsg(tr("Web UI translation for selected locale (%1) has been successfully loaded.")
@@ -370,7 +366,7 @@ void WebApplication::configure()
     m_authSubnetWhitelist = pref->getWebUiAuthSubnetWhitelist();
     m_sessionTimeout = pref->getWebUISessionTimeout();
 
-    m_domainList = pref->getServerDomains().split(';', Qt::SkipEmptyParts);
+    m_domainList = pref->getServerDomains().split(u';', Qt::SkipEmptyParts);
     std::for_each(m_domainList.begin(), m_domainList.end(), [](QString &entry) { entry = entry.trimmed(); });
 
     m_isCSRFProtectionEnabled = pref->isWebUiCSRFProtectionEnabled();
@@ -379,24 +375,24 @@ void WebApplication::configure()
     m_isHttpsEnabled = pref->isWebUiHttpsEnabled();
 
     m_prebuiltHeaders.clear();
-    m_prebuiltHeaders.push_back({QLatin1String(Http::HEADER_X_XSS_PROTECTION), QLatin1String("1; mode=block")});
-    m_prebuiltHeaders.push_back({QLatin1String(Http::HEADER_X_CONTENT_TYPE_OPTIONS), QLatin1String("nosniff")});
+    m_prebuiltHeaders.push_back({Http::HEADER_X_XSS_PROTECTION, u"1; mode=block"_qs});
+    m_prebuiltHeaders.push_back({Http::HEADER_X_CONTENT_TYPE_OPTIONS, u"nosniff"_qs});
 
     if (!m_isAltUIUsed)
-        m_prebuiltHeaders.push_back({QLatin1String(Http::HEADER_REFERRER_POLICY), QLatin1String("same-origin")});
+        m_prebuiltHeaders.push_back({Http::HEADER_REFERRER_POLICY, u"same-origin"_qs});
 
     const bool isClickjackingProtectionEnabled = pref->isWebUiClickjackingProtectionEnabled();
     if (isClickjackingProtectionEnabled)
-        m_prebuiltHeaders.push_back({QLatin1String(Http::HEADER_X_FRAME_OPTIONS), QLatin1String("SAMEORIGIN")});
+        m_prebuiltHeaders.push_back({Http::HEADER_X_FRAME_OPTIONS, u"SAMEORIGIN"_qs});
 
     const QString contentSecurityPolicy =
         (m_isAltUIUsed
-            ? QLatin1String("")
-            : QLatin1String("default-src 'self'; style-src 'self' 'unsafe-inline'; img-src 'self' data:; script-src 'self' 'unsafe-inline'; object-src 'none'; form-action 'self';"))
-        + (isClickjackingProtectionEnabled ? QLatin1String(" frame-ancestors 'self';") : QLatin1String(""))
-        + (m_isHttpsEnabled ? QLatin1String(" upgrade-insecure-requests;") : QLatin1String(""));
+            ? QString()
+            : u"default-src 'self'; style-src 'self' 'unsafe-inline'; img-src 'self' data:; script-src 'self' 'unsafe-inline'; object-src 'none'; form-action 'self';"_qs)
+        + (isClickjackingProtectionEnabled ? u" frame-ancestors 'self';"_qs : QString())
+        + (m_isHttpsEnabled ? u" upgrade-insecure-requests;"_qs : QString());
     if (!contentSecurityPolicy.isEmpty())
-        m_prebuiltHeaders.push_back({QLatin1String(Http::HEADER_CONTENT_SECURITY_POLICY), contentSecurityPolicy});
+        m_prebuiltHeaders.push_back({Http::HEADER_CONTENT_SECURITY_POLICY, contentSecurityPolicy});
 
     if (pref->isWebUICustomHTTPHeadersEnabled())
     {
@@ -405,7 +401,7 @@ void WebApplication::configure()
 
         for (const QStringView line : customHeaderLines)
         {
-            const int idx = line.indexOf(':');
+            const int idx = line.indexOf(u':');
             if (idx < 0)
             {
                 // require separator `:` to be present even if `value` field can be empty
@@ -422,15 +418,29 @@ void WebApplication::configure()
     m_isReverseProxySupportEnabled = pref->isWebUIReverseProxySupportEnabled();
     if (m_isReverseProxySupportEnabled)
     {
+        const QStringList proxyList = pref->getWebUITrustedReverseProxiesList().split(u';', Qt::SkipEmptyParts);
+
         m_trustedReverseProxyList.clear();
+        m_trustedReverseProxyList.reserve(proxyList.size());
 
-        const QStringList proxyList = pref->getWebUITrustedReverseProxiesList().split(';', Qt::SkipEmptyParts);
-
-        for (const QString &proxy : proxyList)
+        for (QString proxy : proxyList)
         {
-            QHostAddress ip;
-            if (ip.setAddress(proxy))
-                m_trustedReverseProxyList.push_back(ip);
+            if (!proxy.contains(u'/'))
+            {
+                const QAbstractSocket::NetworkLayerProtocol protocol = QHostAddress(proxy).protocol();
+                if (protocol == QAbstractSocket::IPv4Protocol)
+                {
+                    proxy.append(u"/32");
+                }
+                else if (protocol == QAbstractSocket::IPv6Protocol)
+                {
+                    proxy.append(u"/128");
+                }
+            }
+
+            const std::optional<Utils::Net::Subnet> subnet = Utils::Net::parseSubnet(proxy);
+            if (subnet)
+                m_trustedReverseProxyList.push_back(subnet.value());
         }
 
         if (m_trustedReverseProxyList.isEmpty())
@@ -438,42 +448,37 @@ void WebApplication::configure()
     }
 }
 
-void WebApplication::registerAPIController(const QString &scope, APIController *controller)
-{
-    Q_ASSERT(controller);
-    Q_ASSERT(!m_apiControllers.value(scope));
-
-    m_apiControllers[scope] = controller;
-}
-
 void WebApplication::declarePublicAPI(const QString &apiPath)
 {
     m_publicAPIs << apiPath;
 }
 
-void WebApplication::sendFile(const QString &path)
+void WebApplication::sendFile(const Path &path)
 {
-    const QDateTime lastModified {QFileInfo(path).lastModified()};
+    const QDateTime lastModified = Utils::Fs::lastModified(path);
 
     // find translated file in cache
-    const auto it = m_translatedFiles.constFind(path);
-    if ((it != m_translatedFiles.constEnd()) && (lastModified <= it->lastModified))
+    if (!m_isAltUIUsed)
     {
-        print(it->data, it->mimeType);
-        setHeader({Http::HEADER_CACHE_CONTROL, getCachingInterval(it->mimeType)});
-        return;
+        if (const auto it = m_translatedFiles.constFind(path);
+            (it != m_translatedFiles.constEnd()) && (lastModified <= it->lastModified))
+        {
+            print(it->data, it->mimeType);
+            setHeader({Http::HEADER_CACHE_CONTROL, getCachingInterval(it->mimeType)});
+            return;
+        }
     }
 
-    QFile file {path};
+    QFile file {path.data()};
     if (!file.open(QIODevice::ReadOnly))
     {
-        qDebug("File %s was not found!", qUtf8Printable(path));
+        qDebug("File %s was not found!", qUtf8Printable(path.toString()));
         throw NotFoundHTTPError();
     }
 
     if (file.size() > MAX_ALLOWED_FILESIZE)
     {
-        qWarning("%s: exceeded the maximum allowed file size!", qUtf8Printable(path));
+        qWarning("%s: exceeded the maximum allowed file size!", qUtf8Printable(path.toString()));
         throw InternalServerErrorHTTPError(tr("Exceeded the maximum allowed file size (%1)!")
                                            .arg(Utils::Misc::friendlyUnit(MAX_ALLOWED_FILESIZE)));
     }
@@ -481,16 +486,20 @@ void WebApplication::sendFile(const QString &path)
     QByteArray data {file.readAll()};
     file.close();
 
-    const QMimeType mimeType {QMimeDatabase().mimeTypeForFileNameAndData(path, data)};
-    const bool isTranslatable {mimeType.inherits(QLatin1String("text/plain"))};
+    const QMimeType mimeType = QMimeDatabase().mimeTypeForFileNameAndData(path.data(), data);
+    const bool isTranslatable = !m_isAltUIUsed && mimeType.inherits(u"text/plain"_qs);
 
-    // Translate the file
     if (isTranslatable)
     {
-        QString dataStr {data};
+        auto dataStr = QString::fromUtf8(data);
+        // Translate the file
         translateDocument(dataStr);
-        data = dataStr.toUtf8();
 
+        // Add the language options
+        if (path == (m_rootFolder / Path(PRIVATE_FOLDER) / Path(u"views/preferences.html"_qs)))
+            dataStr.replace(u"${LANGUAGE_OPTIONS}"_qs, createLanguagesOptionsHtml());
+
+        data = dataStr.toUtf8();
         m_translatedFiles[path] = {data, mimeType.name(), lastModified}; // caching translated file
     }
 
@@ -554,7 +563,7 @@ void WebApplication::sessionInitialize()
 {
     Q_ASSERT(!m_currentSession);
 
-    const QString sessionId {parseCookie(m_request.headers.value(QLatin1String("cookie"))).value(C_SID)};
+    const QString sessionId {parseCookie(m_request.headers.value(u"cookie"_qs)).value(QString::fromLatin1(C_SID))};
 
     // TODO: Additional session check
 
@@ -593,7 +602,7 @@ QString WebApplication::generateSid() const
         const quint32 tmp[] =
         {Utils::Random::rand(), Utils::Random::rand(), Utils::Random::rand()
                 , Utils::Random::rand(), Utils::Random::rand(), Utils::Random::rand()};
-        sid = QByteArray::fromRawData(reinterpret_cast<const char *>(tmp), sizeof(tmp)).toBase64();
+        sid = QString::fromLatin1(QByteArray::fromRawData(reinterpret_cast<const char *>(tmp), sizeof(tmp)).toBase64());
     }
     while (m_sessions.contains(sid));
 
@@ -604,14 +613,14 @@ bool WebApplication::isAuthNeeded()
 {
     if (!m_isLocalAuthEnabled && Utils::Net::isLoopbackAddress(m_clientAddress))
         return false;
-    if (m_isAuthSubnetWhitelistEnabled && Utils::Net::isIPInRange(m_clientAddress, m_authSubnetWhitelist))
+    if (m_isAuthSubnetWhitelistEnabled && Utils::Net::isIPInSubnets(m_clientAddress, m_authSubnetWhitelist))
         return false;
     return true;
 }
 
 bool WebApplication::isPublicAPI(const QString &scope, const QString &action) const
 {
-    return m_publicAPIs.contains(QString::fromLatin1("%1/%2").arg(scope, action));
+    return m_publicAPIs.contains(u"%1/%2"_qs.arg(scope, action));
 }
 
 void WebApplication::sessionStart()
@@ -630,17 +639,24 @@ void WebApplication::sessionStart()
         return false;
     });
 
-    m_currentSession = new WebSession(generateSid());
+    m_currentSession = new WebSession(generateSid(), app());
+    m_currentSession->registerAPIController<AppController>(u"app"_qs);
+    m_currentSession->registerAPIController<LogController>(u"log"_qs);
+    m_currentSession->registerAPIController<RSSController>(u"rss"_qs);
+    m_currentSession->registerAPIController<SearchController>(u"search"_qs);
+    m_currentSession->registerAPIController<SyncController>(u"sync"_qs);
+    m_currentSession->registerAPIController<TorrentsController>(u"torrents"_qs);
+    m_currentSession->registerAPIController<TransferController>(u"transfer"_qs);
     m_sessions[m_currentSession->id()] = m_currentSession;
 
     QNetworkCookie cookie(C_SID, m_currentSession->id().toUtf8());
     cookie.setHttpOnly(true);
     cookie.setSecure(m_isSecureCookieEnabled && m_isHttpsEnabled);
-    cookie.setPath(QLatin1String("/"));
+    cookie.setPath(u"/"_qs);
     QByteArray cookieRawForm = cookie.toRawForm();
     if (m_isCSRFProtectionEnabled)
         cookieRawForm.append("; SameSite=Strict");
-    setHeader({Http::HEADER_SET_COOKIE, cookieRawForm});
+    setHeader({Http::HEADER_SET_COOKIE, QString::fromLatin1(cookieRawForm)});
 }
 
 void WebApplication::sessionEnd()
@@ -648,13 +664,13 @@ void WebApplication::sessionEnd()
     Q_ASSERT(m_currentSession);
 
     QNetworkCookie cookie(C_SID);
-    cookie.setPath(QLatin1String("/"));
+    cookie.setPath(u"/"_qs);
     cookie.setExpirationDate(QDateTime::currentDateTime().addDays(-1));
 
     delete m_sessions.take(m_currentSession->id());
     m_currentSession = nullptr;
 
-    setHeader({Http::HEADER_SET_COOKIE, cookie.toRawForm()});
+    setHeader({Http::HEADER_SET_COOKIE, QString::fromLatin1(cookie.toRawForm())});
 }
 
 bool WebApplication::isCrossSiteRequest(const Http::Request &request) const
@@ -746,7 +762,7 @@ QHostAddress WebApplication::resolveClientAddress() const
         return m_env.clientAddress;
 
     // Only reverse proxy can overwrite client address
-    if (!m_trustedReverseProxyList.contains(m_env.clientAddress))
+    if (!Utils::Net::isIPInSubnets(m_env.clientAddress, m_trustedReverseProxyList))
         return m_env.clientAddress;
 
     const QString forwardedFor = m_request.headers.value(Http::HEADER_X_FORWARDED_FOR);
@@ -754,7 +770,7 @@ QHostAddress WebApplication::resolveClientAddress() const
     if (!forwardedFor.isEmpty())
     {
         // client address is the 1st global IP in X-Forwarded-For or, if none available, the 1st IP in the list
-        const QStringList remoteIpList = forwardedFor.split(',', Qt::SkipEmptyParts);
+        const QStringList remoteIpList = forwardedFor.split(u',', Qt::SkipEmptyParts);
 
         if (!remoteIpList.isEmpty())
         {
@@ -776,8 +792,9 @@ QHostAddress WebApplication::resolveClientAddress() const
 
 // WebSession
 
-WebSession::WebSession(const QString &sid)
-    : m_sid {sid}
+WebSession::WebSession(const QString &sid, IApplication *app)
+    : ApplicationComponent(app)
+    , m_sid {sid}
 {
     updateTimestamp();
 }
@@ -799,12 +816,7 @@ void WebSession::updateTimestamp()
     m_timer.start();
 }
 
-QVariant WebSession::getData(const QString &id) const
+APIController *WebSession::getAPIController(const QString &scope) const
 {
-    return m_data.value(id);
-}
-
-void WebSession::setData(const QString &id, const QVariant &data)
-{
-    m_data[id] = data;
+    return m_apiControllers.value(scope);
 }
